// Интернет-метеостанция SILEN.RU

#include <SPI.h>
#include <Wire.h> //подключение библиотеки интерфейса I2C
#include <LiquidCrystal_I2C.h> //подключение библиотеки экрана 2004
#include <Ethernet.h> //подключение библиотеки вебсервера
#include "DHT.h" //подключение библиотеки датчиков DHT22
#include <Adafruit_BMP085.h> //подключение библиотеки датчика давления BMP085
#include <EEPROM.h> //подключение библиотеки нестираемой памяти часов

#define DHTTYPE DHT22 //описание типа датчика
LiquidCrystal_I2C lcd(0x27,20,4); //определение адреса подключения и типа экрана
#define DS1307_I2C_ADDRESS 0x68 //определение адреса часов
Adafruit_BMP085 bmp;
DHT dht (2, DHTTYPE);
DHT dht2 (3, DHTTYPE);
DHT dht3 (5, DHTTYPE);
DHT dht4 (6, DHTTYPE);
DHT dht5 (7, DHTTYPE);
DHT dht6 (8, DHTTYPE);
DHT dht7 (9, DHTTYPE);
#define DHTPIN 2 //установка pin 1-му датчику DHT22
#define DHTPIN 3 //установка pin 2-му датчику DHT22
#define DHTPIN 5 //установка pin 4-му датчику DHT22
#define DHTPIN 6 //установка pin 4-му датчику DHT22
#define DHTPIN 7 //установка pin 5-му датчику DHT22
#define DHTPIN 8 //установка pin 6-му датчику DHT22
#define DHTPIN 9 //установка pin 6-му датчику DHT22
int led = 19; //назначение светодиоду pin 19

byte mac[] = {0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED}; //назначаем MAC aдрес контроллеру
IPAddress ip(192, 168, 0, 99); //назначаем IP адрес вебсерверу
EthernetServer server(80); //назначаем порт 80 для HTTP

//*****-- //часы
byte setMinClockOn; //
byte setHorClockOn;
byte setMinClockOff; //
byte setHorClockOff;

byte decToBcd(byte val){
  return ( (val/10*16) + (val%10) );
}

byte bcdToDec(byte val){
  return ( (val/16*10) + (val%16) );
}

void setDateDs1307(byte second,        // 0-59
                   byte minute,        // 0-59
                   byte hour,          // 1-23
                   byte dayOfWeek,     // 1-7
                   byte dayOfMonth,    // 1-28/29/30/31
                   byte month,         // 1-12
                   byte year)          // 0-99
{
   Wire.beginTransmission(DS1307_I2C_ADDRESS);
   Wire.write(0);
   Wire.write(decToBcd(second));    
   Wire.write(decToBcd(minute));
   Wire.write(decToBcd(hour));     
   Wire.write(decToBcd(dayOfWeek));
   Wire.write(decToBcd(dayOfMonth));
   Wire.write(decToBcd(month));
   Wire.write(decToBcd(year));
   Wire.endTransmission();
}

void getDateDs1307(byte *second,
          byte *minute,
          byte *hour,
          byte *dayOfWeek,
          byte *dayOfMonth,
          byte *month,
          byte *year)
{

  Wire.beginTransmission(DS1307_I2C_ADDRESS);
  Wire.write(0);
  Wire.endTransmission();
  
  Wire.requestFrom(DS1307_I2C_ADDRESS, 7);

  *second     = bcdToDec(Wire.read() & 0x7f);
  *minute     = bcdToDec(Wire.read());
  *hour       = bcdToDec(Wire.read() & 0x3f);
  *dayOfWeek  = bcdToDec(Wire.read());
  *dayOfMonth = bcdToDec(Wire.read());
  *month      = bcdToDec(Wire.read());
  *year       = bcdToDec(Wire.read());
}

void setClock(){ // установка часов
  byte second, minute, hour, dayOfWeek, dayOfMonth, month, year;
    getDateDs1307(&second, &minute, &hour, &dayOfWeek, &dayOfMonth, &month, &year);
 
   EEPROM.write(0, setMinClockOn);
   EEPROM.write(1, setHorClockOn);
   EEPROM.write(3, setMinClockOff);
   EEPROM.write(4, setHorClockOff);

}
//*****--

void setup() {
Serial.begin(9600); //open serial communications
pinMode(led, OUTPUT); //назначение порту светодиода типа OUTPUT
lcd.init(); //инициализация экрана 
lcd.backlight(); //включение подсветки экрана

  //* заставка при включении
  lcd.setCursor(0, 0);
  lcd.print("        HOME");
  delay(1000);
  lcd.setCursor(0, 1);
  lcd.print("    METEOSTATION");
  delay(1000);
  lcd.setCursor(0, 2);
  lcd.print("      SILEN.RU");
  lcd.setCursor(0, 3);
  delay(1000);
  lcd.print("        2016");
  delay(1000);
  lcd.clear();
 

//** инициализация датчика BMP085
  if (!bmp.begin()) {
    Serial.println("Could not find a valid BMP085 sensor, check wiring!");
    while (1) {}
  }
//**  
  
dht.begin(); //инициализация DHT22

  //start the Ethernet connection and the server:
  Ethernet.begin(mac, ip);
  server.begin();
  Serial.print("server is at ");
  Serial.println(Ethernet.localIP());  
  
  //***//чтение из нестираемой памяти
  setMinClockOn = EEPROM.read(0);
  setHorClockOn = EEPROM.read(1);
  setMinClockOff = EEPROM.read(3);
  setHorClockOff = EEPROM.read(4);
  //***
}

void loop() {
  digitalWrite(led, HIGH); //включение светодиода
  
uint8_t temp_cel[8] = {B00111, B00101, B00111, B00000, B00000, B00000, B00000}; //значок "градус"
lcd.createChar(1, temp_cel); //отрисовка значка "градус Цельсия"
char temp[] = {'\1', 67, '\0'}; //отрисовка значка "градус Цельсия"

byte second, minute, hour, dayOfWeek, dayOfMonth, month, year;  
getDateDs1307(&second, &minute, &hour, &dayOfWeek, &dayOfMonth, &month, &year);
 
  // listen for incoming clients
  EthernetClient client = server.available();
  if (client) {
    Serial.println("new client");
    // an http request ends with a blank line
    boolean currentLineIsBlank = true;
    while (client.connected()) {
      if (client.available()) {
        char c = client.read();
        Serial.write(c);

        // if you've gotten to the end of the line (received a newline
        // character) and the line is blank, the http request has ended,
        // so you can send a reply
        if (c == '\n' && currentLineIsBlank) {
          // send a standard http response header
          client.println("HTTP/1.1 200 OK");
          client.println("Content-Type: text/html");
          client.println("Connection: close");  // the connection will be closed after completion of the response
          client.println("Refresh: 60");  // refresh the page automatically every 5 sec
          client.println();
          client.println("<!DOCTYPE HTML>");
          client.println("<html>");
          client.println("<meta http-equiv='content-type' content='text/html; charset=UTF-8'>");
          client.println("<title>МЕТЕОСТАНЦИЯ</title>");

int t1 = dht.readTemperature(); //снятие показаний температуры 1-го DHT22                    
client.print("<b>УЛИЦА</b><br />t = ");
client.print(t1);
client.print(" °C<br />");

int h1 = dht.readHumidity(); //снятие показаний влажности 1-го DHT22
client.print("h = ");
client.print(h1);
client.print(" %<br /><br />");

int t2 = dht2.readTemperature(); //снятие показаний температуры 2-го DHT22                    
client.print("<b>1 ЭТАЖ</b><br />t = ");
client.print(t2);
client.print(" °C<br />"); 

int h2 = dht2.readHumidity(); //снятие показаний влажности 2-го DHT22
client.print("h = ");
client.print(h2);
client.print(" %<br /><br />");

int t3 = dht3.readTemperature(); //снятие показаний температуры 3-го DHT22                    
client.print("<b>2 ЭТАЖ</b><br />t = ");
client.print(t3);
client.print(" °C<br />");  

int h3 = dht3.readHumidity(); //снятие показаний влажности 3-го DHT22
client.print("h = ");
client.print(h3);
client.print(" %<br /><br />");

int t4 = dht4.readTemperature(); //снятие показаний температуры 4-го DHT22                    
client.print("<b>3 ЭТАЖ</b><br />t = ");
client.print(t4);
client.print(" °C<br />");  

int h4 = dht4.readHumidity(); //снятие показаний влажности 4-го DHT22
client.print("h = ");
client.print(h4);
client.print(" %<br /><br />");

int t5 = dht5.readTemperature(); //снятие показаний температуры 5-го DHT22                    
client.print("<b>ЦОКОЛЬ</b><br />t = ");
client.print(t5);
client.print(" °C<br />");  

int h5 = dht5.readHumidity(); //снятие показаний влажности 5-го DHT22
client.print("h = ");
client.print(h5);
client.print(" %");
client.println("<br /><br />");

int t6 = dht6.readTemperature(); //снятие показаний температуры 6-го DHT22                    
client.print("<b>ХРАНИЛИЩЕ</b><br />t = ");
client.print(t6);
client.print(" °C<br />"); 

int h6 = dht6.readHumidity(); //снятие показаний влажности 6-го DHT22
client.print("h = ");
client.print(h6);
client.print(" %<br /><br />");

int t7 = dht7.readTemperature(); //снятие показаний температуры 7-го DHT22                    
client.print("<b>ГАРАЖ</b><br />t = ");
client.print(t7);
client.print(" °C<br />"); 

int h7 = dht7.readHumidity(); //снятие показаний влажности 7-го DHT22
client.print("h = ");
client.print(h7);
client.print(" %<br /><br />"); 

client.print("<b>ДАВЛЕНИЕ</b><br />p = ");
client.print(int(bmp.readPressure()/133.322));
client.print(" мм.рт.ст.<br /><br />");

          client.println("</html>");
          break;
        }
        if (c == '\n') {
          // you're starting a new line
          currentLineIsBlank = true;
        }
        else if (c != '\r') {
          // you've gotten a character on the current line
          currentLineIsBlank = false;
        }
      }
    }
    // give the web browser time to receive the data
    delay(1);
    // close the connection:
    client.stop();
    Serial.println("client disconnected");
  }

int t1 = dht.readTemperature(); //снятие показаний температуры с 1-го DHT22
int h1 = dht.readHumidity(); //снятие показаний влажности с 1-го DHT22
int t2 = dht2.readTemperature(); //снятие показаний температуры с 2-го DHT22
int h2 = dht2.readHumidity(); //снятие показаний влажности с 2-го DHT22

lcd.setCursor(0, 0); //установка курсора на экране
if (hour < 10) lcd.print("0");
lcd.print(hour); //вывод на экран часы
lcd.print(":"); //вывод на экран ":"
if (minute < 10) lcd.print("0");
lcd.print(minute); //вывод на экран минуты

lcd.setCursor(10,0); //установка курсора на экране
if (dayOfMonth < 10) lcd.print("0");
lcd.print(dayOfMonth);
lcd.print("."); //вывод на экран "."
if (month < 10) lcd.print("0");
lcd.print(month);
lcd.print(".20"); //вывод на экран ".20"
if (year < 10) lcd.print("0");
lcd.print(year);

lcd.setCursor(0,1); //установка курсора на экране
lcd.print("----- P: "); //вывод на экран
lcd.print(int(bmp.readPressure()/133.322)); //вывод на экран показания атмосферного давления с BMP085
lcd.print("mm -----"); //вывод на экран "mm"

lcd.setCursor(0,2); //установка курсора на экране
lcd.print("OUT  T:"); //вывод на экран
lcd.setCursor(8,2); //установка курсора на экране
lcd.print(t1); //вывод на экран показания температуры с первого DHT22
lcd.setCursor(10,2); //установка курсора на экране
lcd.print(temp); //вывод на экран значка "градус Цельсия"

lcd.setCursor(14,2); //установка курсора на экране
lcd.print("H:"); //вывод на экран
lcd.setCursor(17,2); //установка курсора на экране
lcd.print(h1); //вывод на экран показания влажности с первого DHT22
lcd.setCursor(19,2); //установка курсора на экране
lcd.print("%"); //вывод на экран

lcd.setCursor(0,3); //установка курсора на экране
lcd.print("IN   T:"); //вывод на экран
lcd.setCursor(8,3); //установка курсора на экране
lcd.print(t2); //вывод на экран показания температуры со второго DHT22
lcd.setCursor(10,3); //установка курсора на экране
lcd.print(temp); //вывод на экран значка "градус Цельсия"

lcd.setCursor(14,3); //установка курсора на экране
lcd.print("H:"); //вывод на экран
lcd.setCursor(17,3); //установка курсора на экране
lcd.print(h2); //вывод на экран показания влажности со второго DHT22
lcd.setCursor(19,3); //установка курсора на экране
lcd.print("%"); //вывод на экран

delay(10000);
}
